@@ -22,11 +22,17 @@ typedef struct {
 	gpointer           io_data;
 	SoupMessageIOStatus io_status;
 
-	guint              msg_flags;
+	SoupChunkAllocator chunk_allocator;
+	gpointer           chunk_allocator_data;
+	GDestroyNotify     chunk_allocator_dnotify;
 
-	SoupHTTPVersion    http_version;
+	guint              msg_flags;
+	gboolean           server_side;
+
+	SoupHTTPVersion    http_version, orig_http_version;
 
 	SoupURI           *uri;
+	SoupAddress       *addr;
 
 	SoupAuth          *auth, *proxy_auth;
 } SoupMessagePrivate;
